package com.example.demo.orders.api;

import java.sql.Date;
import java.util.List;
import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.*;
import com.example.demo.core.configuration.Constants;
import com.example.demo.order_lines.api.OrderLineDto;
import com.example.demo.order_lines.model.OrderLineEntity;
import com.example.demo.orders.model.OrderEntity;
import com.example.demo.orders.service.OrderService;
import com.example.demo.products.model.ProductEntity;
import com.example.demo.products.service.ProductService;
import com.example.demo.users.service.UserService;
import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/order")
public class OrderController {
    private final OrderService orderService;
    private final UserService userService;
    private final ModelMapper modelMapper;
    private final ProductService productService;

    public OrderController(OrderService orderService, UserService userService, ModelMapper modelMapper,
            ProductService productService) {
        this.orderService = orderService;
        this.userService = userService;
        this.productService = productService;
        this.modelMapper = modelMapper;
    }

    private OrderDto toDto(OrderEntity entity) {
        var dto = new OrderDto();
        dto.setId(entity.getId());
        dto.setUserId(entity.getUser().getId());
        dto.setDate(new Date(System.currentTimeMillis()));
        for (OrderLineEntity orderLineEntity : entity.getLines()) {
            OrderLineDto orderLineDto = new OrderLineDto();
            orderLineDto.setProductId(orderLineEntity.getProduct().getId());
            orderLineDto.setCount(orderLineEntity.getCount());
            // Вычисляем общую цену строки заказа
            Double totalPriceLine = orderLineEntity.getProduct().getPrice() * orderLineEntity.getCount();
            orderLineDto.setTotalPriceLine(totalPriceLine);
            dto.addOrderLine(orderLineDto);
        }
        return dto;
    }

    public OrderEntity toEntity(OrderDto dto) {
        final OrderEntity entity = modelMapper.map(dto, OrderEntity.class);
        if (dto.getUserId() != null) {
            entity.setUser(userService.get(dto.getUserId()));
        }
        entity.getLines().clear();
        for (OrderLineDto lineDto : dto.getLines()) {
            OrderLineEntity orderLineEntity = modelMapper.map(lineDto, OrderLineEntity.class);
            // Здесь нужно установить ProductEntity для OrderLineEntity
            ProductEntity product = productService.get(lineDto.getProductId()); // Получаем продукт по его ID
            orderLineEntity.setProduct(product); // Устанавливаем продукт для строки заказа
            entity.addOrderLine(orderLineEntity);
        }
        return entity;
    }

    @GetMapping
    public List<OrderDto> getAll(@RequestParam(name = "userId", defaultValue = "0") Long userId,
            @RequestParam(name = "lines", defaultValue = "") List<Long> lines) {
        return orderService.getAll(userId, lines).stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public OrderDto get(@PathVariable(name = "id") Long id) {
        return toDto(orderService.get(id));
    }

    @PostMapping
    public OrderDto create(@RequestBody @Valid OrderDto dto) {
        OrderEntity orderEntity = toEntity(dto);
        return toDto(orderService.create(orderEntity));
    }

    @PutMapping("/{id}")
    public OrderDto update(@PathVariable(name = "id") Long id, @RequestBody OrderDto dto) {
        return toDto(orderService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public OrderDto delete(@PathVariable(name = "id") Long id) {
        return toDto(orderService.delete(id));
    }
}
